@@ -28,7 +28,7 @@
 #ifndef HAVE_BACON_VIDEO_WIDGET_H
 #define HAVE_BACON_VIDEO_WIDGET_H
 
-#include <gtk/gtk.h>
+#include <clutter-gtk/clutter-gtk.h>
 /* for optical disc enumeration type */
 #include <totem-disc.h>
 
@@ -51,7 +51,7 @@ typedef struct BaconVideoWidgetCommon BaconVideoWidgetCommon;
  **/
 typedef struct {
 	/*< private >*/
-	GtkEventBox parent;
+	GtkClutterEmbed parent;
 	BaconVideoWidgetPrivate *priv;
 } BaconVideoWidget;
 
@@ -62,7 +62,7 @@ typedef struct {
  **/
 typedef struct {
 	/*< private >*/
-	GtkEventBoxClass parent_class;
+	GtkClutterEmbedClass parent_class;
 
 	void (*error) (GtkWidget *bvw, const char *message,
                        gboolean playback_stopped, gboolean fatal);
@@ -139,30 +139,8 @@ typedef enum {
 GQuark bacon_video_widget_error_quark		 (void) G_GNUC_CONST;
 GType bacon_video_widget_get_type                (void);
 GOptionGroup* bacon_video_widget_get_option_group (void);
-/* This can be used if the app does not use popt */
-void bacon_video_widget_init_backend		 (int *argc, char ***argv);
 
-/**
- * BvwUseType:
- * @BVW_USE_TYPE_VIDEO: fully-featured with video, audio, capture and metadata support
- * @BVW_USE_TYPE_AUDIO: audio and metadata support
- * @BVW_USE_TYPE_CAPTURE: capture support only
- * @BVW_USE_TYPE_METADATA: metadata support only
- *
- * The purpose for which a #BaconVideoWidget will be used, as specified to
- * bacon_video_widget_new(). This determines which features will be enabled
- * in the created widget.
- **/
-typedef enum {
-	BVW_USE_TYPE_VIDEO,
-	BVW_USE_TYPE_AUDIO,
-	BVW_USE_TYPE_CAPTURE,
-	BVW_USE_TYPE_METADATA
-} BvwUseType;
-
-GtkWidget *bacon_video_widget_new		 (int width, int height,
-						  BvwUseType type,
-						  GError **error);
+GtkWidget *bacon_video_widget_new		 (GError **error);
 
 char *bacon_video_widget_get_backend_name (BaconVideoWidget *bvw);
 
@@ -306,6 +284,7 @@ typedef enum {
 	BVW_INFO_DURATION,
 	BVW_INFO_TRACK_NUMBER,
 	BVW_INFO_COVER,
+	BVW_INFO_CONTAINER,
 	/* Video */
 	BVW_INFO_HAS_VIDEO,
 	BVW_INFO_DIMENSION_X,
@@ -388,6 +367,19 @@ typedef enum {
 	BVW_RATIO_DVB = 4
 } BvwAspectRatio;
 
+/**
+ * BvwZoomMode:
+ * @BVW_ZOOM_NONE: No video zooming/cropping
+ * @BVW_ZOOM_EXPAND: Fill area with video, and crop the excess
+ *
+ * The zoom mode used by the video widget, as set by
+ * bacon_video_widget_set_zoom().
+ **/
+typedef enum {
+	BVW_ZOOM_NONE = 0,
+	BVW_ZOOM_EXPAND = 1
+} BvwZoomMode;
+
 void bacon_video_widget_set_deinterlacing        (BaconVideoWidget *bvw,
 						  gboolean deinterlace);
 gboolean bacon_video_widget_get_deinterlacing    (BaconVideoWidget *bvw);
@@ -401,8 +393,8 @@ void bacon_video_widget_set_scale_ratio          (BaconVideoWidget *bvw,
 						  float ratio);
 
 void bacon_video_widget_set_zoom		 (BaconVideoWidget *bvw,
-						  double zoom);
-double bacon_video_widget_get_zoom		 (BaconVideoWidget *bvw);
+						  BvwZoomMode       mode);
+BvwZoomMode bacon_video_widget_get_zoom		 (BaconVideoWidget *bvw);
 
 int bacon_video_widget_get_video_property        (BaconVideoWidget *bvw,
 						  BvwVideoProperty type);
@@ -410,7 +402,10 @@ void bacon_video_widget_set_video_property       (BaconVideoWidget *bvw,
 						  BvwVideoProperty type,
 						  int value);
 
+void bacon_video_widget_set_next_angle           (BaconVideoWidget *bvw);
+
 gboolean bacon_video_widget_has_menus            (BaconVideoWidget *bvw);
+gboolean bacon_video_widget_has_angles           (BaconVideoWidget *bvw);
 
 /* DVD functions */
 /**
@@ -425,8 +420,6 @@ gboolean bacon_video_widget_has_menus            (BaconVideoWidget *bvw);
  * @BVW_DVD_PREV_CHAPTER: the previous chapter
  * @BVW_DVD_NEXT_TITLE: the next title in the current chapter
  * @BVW_DVD_PREV_TITLE: the previous title in the current chapter
- * @BVW_DVD_NEXT_ANGLE: the next angle
- * @BVW_DVD_PREV_ANGLE: the previous angle
  * @BVW_DVD_ROOT_MENU_UP: go up in the menu
  * @BVW_DVD_ROOT_MENU_DOWN: go down in the menu
  * @BVW_DVD_ROOT_MENU_LEFT: go left in the menu
@@ -447,8 +440,6 @@ typedef enum {
 	BVW_DVD_PREV_CHAPTER,
 	BVW_DVD_NEXT_TITLE,
 	BVW_DVD_PREV_TITLE,
-	BVW_DVD_NEXT_ANGLE,
-	BVW_DVD_PREV_ANGLE,
 	BVW_DVD_ROOT_MENU_UP,
 	BVW_DVD_ROOT_MENU_DOWN,
 	BVW_DVD_ROOT_MENU_LEFT,
