@@ -36,8 +36,11 @@
 #include "totem-interface.h"
 #include "totem-time-label.h"
 #include "bacon-video-widget.h"
+#include "gsd-media-keys-window.h"
 
 #define FULLSCREEN_POPUP_TIMEOUT 5
+#define FULLSCREEN_MOTION_TIME 200 /* in milliseconds */
+#define FULLSCREEN_MOTION_NUM_EVENTS 15
 
 static void totem_fullscreen_dispose (GObject *object);
 static void totem_fullscreen_finalize (GObject *object);
@@ -49,10 +52,14 @@ G_MODULE_EXPORT gboolean totem_fullscreen_vol_slider_released_cb (GtkWidget *wid
 G_MODULE_EXPORT gboolean totem_fullscreen_seek_slider_pressed_cb (GtkWidget *widget, GdkEventButton *event, TotemFullscreen *fs);
 G_MODULE_EXPORT gboolean totem_fullscreen_seek_slider_released_cb (GtkWidget *widget, GdkEventButton *event, TotemFullscreen *fs);
 G_MODULE_EXPORT gboolean totem_fullscreen_motion_notify (GtkWidget *widget, GdkEventMotion *event, TotemFullscreen *fs);
+G_MODULE_EXPORT gboolean totem_fullscreen_control_enter_notify (GtkWidget *widget, GdkEventCrossing *event, TotemFullscreen *fs);
+G_MODULE_EXPORT gboolean totem_fullscreen_control_leave_notify (GtkWidget *widget, GdkEventCrossing *event, TotemFullscreen *fs);
+
 
 struct _TotemFullscreenPrivate {
 	BaconVideoWidget *bvw;
 	GtkWidget        *parent_window;
+	GtkWidget        *osd;
 
 	/* Fullscreen Popups */
 	GtkWidget        *exit_popup;
@@ -63,7 +70,10 @@ struct _TotemFullscreenPrivate {
 
 	guint             popup_timeout;
 	gboolean          popup_in_progress;
+	gboolean          pointer_on_control;
 	guint             motion_handler_id;
+	guint             motion_start_time;
+	guint             motion_num_events;
 
 	GtkBuilder       *xml;
 };
@@ -88,14 +98,16 @@ totem_fullscreen_move_popups (TotemFullscreen *fs)
 	
 	GdkScreen              *screen;
 	GdkRectangle            fullscreen_rect;
+	GdkWindow              *window;
 	TotemFullscreenPrivate *priv = fs->priv;
 
 	g_return_if_fail (priv->parent_window != NULL);
 
 	/* Obtain the screen rectangle */
 	screen = gtk_window_get_screen (GTK_WINDOW (priv->parent_window));
+	window = gtk_widget_get_window (priv->parent_window);
 	gdk_screen_get_monitor_geometry (screen,
-					 gdk_screen_get_monitor_at_window (screen, priv->parent_window->window),
+					 gdk_screen_get_monitor_at_window (screen, window),
 					 &fullscreen_rect);
 
 	/* Get the popup window sizes */
@@ -140,6 +152,20 @@ totem_fullscreen_theme_changed_cb (GtkIconTheme *icon_theme, TotemFullscreen *fs
 	totem_fullscreen_move_popups (fs);
 }
 
+static void
+totem_fullscreen_composited_changed_cb (GdkScreen *screen, TotemFullscreen *fs)
+{
+	if (gdk_screen_is_composited (screen)) {
+		if (fs->priv->osd == NULL)
+			fs->priv->osd = gsd_media_keys_window_new ();
+	} else {
+		if (fs->priv->osd != NULL) {
+			gtk_widget_destroy (fs->priv->osd);
+			fs->priv->osd = NULL;
+		}
+	}
+}
+
 static void
 totem_fullscreen_window_realize_cb (GtkWidget *widget, TotemFullscreen *fs)
 {
@@ -148,9 +174,13 @@ totem_fullscreen_window_realize_cb (GtkWidget *widget, TotemFullscreen *fs)
 	screen = gtk_widget_get_screen (widget);
 	g_signal_connect (G_OBJECT (screen), "size-changed",
 			  G_CALLBACK (totem_fullscreen_size_changed_cb), fs);
+	g_signal_connect (G_OBJECT (screen), "composited-changed",
+			  G_CALLBACK (totem_fullscreen_composited_changed_cb), fs);
 	g_signal_connect (G_OBJECT (gtk_icon_theme_get_for_screen (screen)),
 			  "changed",
 			  G_CALLBACK (totem_fullscreen_theme_changed_cb), fs);
+
+	totem_fullscreen_composited_changed_cb (screen, fs);
 }
 
 static void
@@ -165,6 +195,26 @@ totem_fullscreen_window_unrealize_cb (GtkWidget *widget, TotemFullscreen *fs)
 					      G_CALLBACK (totem_fullscreen_theme_changed_cb), fs);
 }
 
+static gboolean
+totem_fullscreen_exit_popup_draw_cb (GtkWidget *widget,
+				     cairo_t *cr,
+				     TotemFullscreen *fs)
+{
+	GdkScreen *screen;
+
+	screen = gtk_widget_get_screen (widget);
+	if (gdk_screen_is_composited (screen) == FALSE)
+		return FALSE;
+
+	gtk_widget_set_app_paintable (widget, TRUE);
+
+	cairo_set_source_rgba (cr, 1., 1., 1., 0.);
+	cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
+	cairo_paint (cr);
+
+	return FALSE;
+}
+
 gboolean
 totem_fullscreen_seek_slider_pressed_cb (GtkWidget *widget,
 					 GdkEventButton *event,
@@ -209,7 +259,7 @@ totem_fullscreen_set_cursor (TotemFullscreen *fs, gboolean state)
 static gboolean
 totem_fullscreen_is_volume_popup_visible (TotemFullscreen *fs)
 {
-	return GTK_WIDGET_VISIBLE (gtk_scale_button_get_popup (GTK_SCALE_BUTTON (fs->volume)));
+	return gtk_widget_get_visible (gtk_scale_button_get_popup (GTK_SCALE_BUTTON (fs->volume)));
 }
 
 static void
@@ -217,7 +267,7 @@ totem_fullscreen_force_popup_hide (TotemFullscreen *fs)
 {
 	/* Popdown the volume button if it's visible */
 	if (totem_fullscreen_is_volume_popup_visible (fs))
-		gtk_bindings_activate (GTK_OBJECT (fs->volume), GDK_Escape, 0);
+		gtk_bindings_activate (G_OBJECT (fs->volume), GDK_KEY_Escape, 0);
 
 	gtk_widget_hide (fs->priv->exit_popup);
 	gtk_widget_hide (fs->priv->control_popup);
@@ -242,10 +292,32 @@ totem_fullscreen_popup_hide (TotemFullscreen *fs)
 }
 
 G_MODULE_EXPORT gboolean
-totem_fullscreen_motion_notify (GtkWidget *widget, GdkEventMotion *event,
+totem_fullscreen_motion_notify (GtkWidget *widget,
+				GdkEventMotion *event,
 				TotemFullscreen *fs)
 {
-	totem_fullscreen_show_popups (fs, TRUE);
+	int motion_delay;
+
+	/* Only after FULLSCREEN_MOTION_NUM_EVENTS motion events,
+	   in FULLSCREEN_MOTION_TIME milliseconds will we show
+	   the popups */
+	motion_delay = event->time - fs->priv->motion_start_time;
+
+	if (fs->priv->motion_start_time == 0 ||
+	    motion_delay < 0 ||
+	    motion_delay > FULLSCREEN_MOTION_TIME) {
+		fs->priv->motion_start_time = event->time;
+		fs->priv->motion_num_events = 0;
+		return FALSE;
+	}
+
+	fs->priv->motion_num_events++;
+
+	if (!fs->priv->pointer_on_control &&
+	    fs->priv->motion_num_events > FULLSCREEN_MOTION_NUM_EVENTS) {
+		totem_fullscreen_show_popups (fs, TRUE);
+	}
+
 	return FALSE;
 }
 
@@ -266,7 +338,7 @@ totem_fullscreen_show_popups (TotemFullscreen *fs, gboolean show_cursor)
 
 	totem_fullscreen_popup_timeout_remove (fs);
 
-	/* FIXME: is this really required while we are anyway going 
+	/* FIXME: is this really required while we are anyway going
 	   to do a show_all on its parent control_popup? */
 	item = GTK_WIDGET (gtk_builder_get_object (fs->priv->xml, "tcw_hbox"));
 	gtk_widget_show_all (item);
@@ -288,6 +360,63 @@ totem_fullscreen_show_popups (TotemFullscreen *fs, gboolean show_cursor)
 	fs->priv->popup_in_progress = FALSE;
 }
 
+void
+totem_fullscreen_show_popups_or_osd (TotemFullscreen *fs,
+				     const char *icon_name,
+				     gboolean show_cursor)
+{
+	GtkAllocation allocation;
+	GdkScreen *screen;
+	GdkWindow *window;
+	GdkRectangle rect;
+	int monitor;
+
+	if (fs->priv->osd == NULL || icon_name == NULL) {
+		totem_fullscreen_show_popups (fs, show_cursor);
+		return;
+	}
+
+	gtk_widget_get_allocation (GTK_WIDGET (fs->priv->bvw), &allocation);
+	gtk_window_resize (GTK_WINDOW (fs->priv->osd),
+			   allocation.height / 8,
+			   allocation.height / 8);
+
+	window = gtk_widget_get_window (GTK_WIDGET (fs->priv->bvw));
+	screen = gtk_widget_get_screen (GTK_WIDGET (fs->priv->bvw));
+	monitor = gdk_screen_get_monitor_at_window (screen, window);
+	gdk_screen_get_monitor_geometry (screen, monitor, &rect);
+
+	if (gtk_widget_get_direction (GTK_WIDGET (fs->priv->bvw)) == GTK_TEXT_DIR_RTL)
+		gtk_window_move (GTK_WINDOW (fs->priv->osd),
+				 rect.width - 8 - allocation.height / 8,
+				 rect.y + 8);
+	else
+		gtk_window_move (GTK_WINDOW (fs->priv->osd), rect.x + 8, rect.y + 8);
+
+	gsd_media_keys_window_set_action_custom (GSD_MEDIA_KEYS_WINDOW (fs->priv->osd),
+						 icon_name, FALSE);
+	gtk_widget_show (fs->priv->osd);
+}
+
+G_MODULE_EXPORT gboolean
+totem_fullscreen_control_enter_notify (GtkWidget *widget,
+			       GdkEventCrossing *event,
+			       TotemFullscreen *fs)
+{
+	fs->priv->pointer_on_control = TRUE;
+	totem_fullscreen_popup_timeout_remove (fs);
+	return TRUE;
+}
+
+G_MODULE_EXPORT gboolean
+totem_fullscreen_control_leave_notify (GtkWidget *widget,
+			       GdkEventCrossing *event,
+			       TotemFullscreen *fs)
+{
+	fs->priv->pointer_on_control = FALSE;
+	return TRUE;
+}
+
 void
 totem_fullscreen_set_fullscreen (TotemFullscreen *fs,
 				 gboolean fullscreen)
@@ -298,6 +427,8 @@ totem_fullscreen_set_fullscreen (TotemFullscreen *fs,
 
 	bacon_video_widget_set_fullscreen (fs->priv->bvw, fullscreen);
 	totem_fullscreen_set_cursor (fs, !fullscreen);
+	if (fullscreen == FALSE)
+		gtk_widget_hide (fs->priv->osd);
 
 	fs->is_fullscreen = fullscreen;
 
@@ -324,7 +455,7 @@ totem_fullscreen_parent_window_notify (GtkWidget *parent_window,
 	popup = gtk_scale_button_get_popup (GTK_SCALE_BUTTON (fs->volume));
 	if (parent_window == fs->priv->parent_window &&
 	    gtk_window_is_active (GTK_WINDOW (parent_window)) == FALSE &&
-	    GTK_WIDGET_VISIBLE (popup) == FALSE) {
+	    gtk_widget_get_visible (popup) == FALSE) {
 		totem_fullscreen_force_popup_hide (fs);
 		totem_fullscreen_set_cursor (fs, TRUE);
 	} else {
@@ -410,8 +541,12 @@ totem_fullscreen_init (TotemFullscreen *self)
 	if (self->priv->xml == NULL)
 		return;
 
+	self->priv->pointer_on_control = FALSE;
+
 	self->priv->exit_popup = GTK_WIDGET (gtk_builder_get_object (self->priv->xml,
 				"totem_exit_fullscreen_window"));
+	g_signal_connect (G_OBJECT (self->priv->exit_popup), "draw",
+			  G_CALLBACK (totem_fullscreen_exit_popup_draw_cb), self);
 	self->priv->control_popup = GTK_WIDGET (gtk_builder_get_object (self->priv->xml,
 				"totem_controls_window"));
 
