@@ -28,13 +28,11 @@
 #ifndef __TOTEM_PRIVATE_H__
 #define __TOTEM_PRIVATE_H__
 
-#include <gconf/gconf-client.h>
 #include <gtk/gtk.h>
 #include <gio/gio.h>
-#include <unique/uniqueapp.h>
 
 #include "totem-playlist.h"
-#include "bacon-video-widget.h"
+#include "backend/bacon-video-widget.h"
 #include "totem-open-location.h"
 #include "totem-fullscreen.h"
 #include "totem-plugins-engine.h"
@@ -82,7 +80,6 @@ struct _TotemObject {
 
 	/* UI manager */
 	GtkActionGroup *main_action_group;
-	GtkActionGroup *zoom_action_group;
 	GtkUIManager *ui_manager;
 
 	GtkActionGroup *devices_action_group;
@@ -159,8 +156,8 @@ struct _TotemObject {
 	char *mrl;
 	gint64 seek_to;
 	TotemPlaylist *playlist;
-	GConfClient *gc;
-	UniqueApp *app;
+	GSettings *settings;
+	GtkApplication *app;
 	TotemStates state;
 	TotemOpenLocation *open_location;
 	gboolean remember_position;
@@ -175,14 +172,16 @@ GtkWidget *totem_volume_create (void);
 #define VOLUME_DOWN_OFFSET (-0.08)
 #define VOLUME_UP_OFFSET (0.08)
 
+#define VOLUME_DOWN_SHORT_OFFSET (-0.02)
+#define VOLUME_UP_SHORT_OFFSET (0.02)
+
 #define ZOOM_IN_OFFSET 0.01
 #define ZOOM_OUT_OFFSET -0.01
 
 void	totem_action_open			(Totem *totem);
 void	totem_action_open_location		(Totem *totem);
 void	totem_action_eject			(Totem *totem);
-void	totem_action_zoom_relative		(Totem *totem, double off_pct);
-void	totem_action_zoom_reset			(Totem *totem);
+void	totem_action_set_zoom			(Totem *totem, gboolean zoom);
 void	totem_action_show_help			(Totem *totem);
 void	totem_action_show_properties		(Totem *totem);
 gboolean totem_action_open_files		(Totem *totem, char **list);
